require 'rails_helper'

module Mks
  module Rent
    RSpec.describe OfferRequestsController, type: :controller do
      routes { Mks::Rent::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"

        @client = create(:client)
        @b1 = create(:service_bundle)
        @b2 = create(:service_bundle)
      end

      let(:valid_attributes) do
        {
          client: {
            id: @client.id,
            name: @client.name
          },
          destination: FFaker::Name.name,
          description: FFaker::Name.name,
          items: [
            { service_bundle_id: @b1.id, from: Date.today, to: Date.today + 10, quantity: 1 },
            { service_bundle_id: @b2.id, from: Date.today, to: Date.today + 10, quantity: 1 }
          ]
        }
      end

      let(:invalid_attributes) do
        {
          client: {
            id: nil,
            name: nil
          },
          destination: FFaker::Name.name,
          description: FFaker::Name.name
        }
      end

      describe 'GET #index' do
        it 'returns a success response' do
          create(:offer_request)
          get :index
          expect(response).to be_success
        end
      end

      describe 'GET #show' do
        it 'returns a success response' do
          offer_request = create(:offer_request)
          get :show, params: { id: offer_request.to_param }
          expect(response).to be_success
        end

        it 'serializes client and items' do
          user = create(:user)

          cl = create(:client)
          request = create(:offer_request, client: cl, prepared_by: user)
          2.times { create(:request_item, offer_request: request) }
          get :show, params: { id: request.to_param }
          result = JSON(response.body)
          expect(result['data']['client']['id']).to eq cl.id
          expect(result['data']['request_items'].count).to eq 2
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new OfferRequest' do
            request = create(:offer_request)
            expect_any_instance_of(OfferRequestService).to receive(:create).and_return([request, true])
            post :create, params: { offer_request: valid_attributes }
            result = JSON(response.body)
            expect(result['data']['id']).to eq request.id
          end

          it 'renders a JSON response with the new offer_request' do
            request = create(:offer_request)
            expect_any_instance_of(OfferRequestService).to receive(:create).and_return([request, true])
            post :create, params: { offer_request: valid_attributes }
            expect(response).to have_http_status(:created)
            expect(response.content_type).to eq('application/json')
            expect(response.location).to eq(offer_request_url(OfferRequest.last))
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the new offer_request' do
            post :create, params: { offer_request: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end

    end
  end
end