require 'rails_helper'

module Mks
  module Rent
    RSpec.describe OfferRequestService do
      before(:all) do
        @service = OfferRequestService.new
      end

      describe '#get_or_create_client' do
        it 'returns client id for existing client' do
          client = create(:client)
          params = { id: client.id, name: client.name }
          id = @service.get_or_create_client(params)
          expect(id).to eq client.id
        end

        it 'creates a new client if no id exists' do
          params = { name: FFaker::Name.name }
          id = @service.get_or_create_client(params)
          client = Client.find(id)
          expect(client.name).to eq params[:name]
        end
      end

      describe '#create' do
        it 'creates offer request for an existing client' do
          client = create(:client)
          user = create(:user)
          eq1 = create(:service_bundle)
          eq2 = create(:service_bundle)

          params = {
            description: FFaker::Name.name,
            destination: FFaker::Name.name,
            client: {
              id: client.id,
              name: client.name
            },
            items: [
              { service_bundle_id: eq1.id, from: Date.today, to: Date.today + 5, quantity: 1 },
              { service_bundle_id: eq2.id, from: Date.today, to: Date.today + 5, quantity: 1 }
            ]
          }

          request, saved = @service.create(params, user)
          expect(saved).to be_truthy
          expect(request.prepared_by).to eq user
          expect(request.destination).to eq params[:destination]
          expect(request.description).to eq params[:description]

        end

        it 'creates offer request for a new client' do
          user = create(:user)
          eq1 = create(:service_bundle)
          eq2 = create(:service_bundle)
          params = {
            description: FFaker::Name.name,
            destination: FFaker::Name.name,
            client: {
              name: FFaker::Name.name
            },
            items: [
              { service_bundle_id: eq1.id, from: Date.today, to: Date.today + 5, quantity: 1 },
              { service_bundle_id: eq2.id, from: Date.today, to: Date.today + 5, quantity: 1 }
            ]
          }

          request, saved = @service.create(params, user)
          expect(saved).to be_truthy
          expect(request.prepared_by).to eq user
          expect(request.destination).to eq params[:destination]
          expect(request.description).to eq params[:description]
          expect(request.client.name).to eq params[:client][:name]
        end

        it 'raises error if no item is supplied' do
          client = create(:client)
          user = create(:user)

          params = {
            description: FFaker::Name.name,
            destination: FFaker::Name.name,
            client: {
              id: client.id,
              name: client.name
            }
          }

          request, saved = @service.create(params, user)
          expect(saved).to be_falsey
          expect(request.errors.messages[:base]).to include 'Request has no items'

        end
      end

      describe '#update' do
        it 'updates offer request' do
          client = create(:client)
          offer_request = create(:offer_request)

          params = {
            client: {
              id: client.id,
              name: client.name
            },
            description: FFaker::Name.name,
            destination: FFaker::Name.name
          }

          @service.update(offer_request, params)
          offer_request.reload

          expect(offer_request.client_id).to eq client.id
          expect(offer_request.description).to eq params[:description]
          expect(offer_request.destination).to eq params[:destination]
        end

        it 'updates request with a new client' do
          offer_request = create(:offer_request)

          params = {
            client: {
              name: FFaker::Name.name
            }
          }

          @service.update(offer_request, params)
          offer_request.reload

          expect(offer_request.client.name).to eq params[:client][:name]
        end
      end
    end
  end
end

